"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseAssertion = void 0;
const node_process_1 = __importDefault(require("process"));
const node_path_1 = __importDefault(require("path"));
const node_assert_1 = __importDefault(require("assert"));
const node_os_1 = require("os");
const format_js_1 = require("./format.js");
const hr_js_1 = require("./hr.js");
const find_file_js_1 = require("./find-file.js");
const each_js_1 = require("../configs/each.js");
const parseAssertion = (cb, options) => __awaiter(void 0, void 0, void 0, function* () {
    var _a, _b;
    const isPoku = typeof ((_a = node_process_1.default.env) === null || _a === void 0 ? void 0 : _a.FILE) === 'string' && ((_b = node_process_1.default.env) === null || _b === void 0 ? void 0 : _b.FILE.length) > 0;
    const FILE = node_process_1.default.env.FILE;
    try {
        if (typeof each_js_1.each.before.cb === 'function' && each_js_1.each.before.assert) {
            const beforeResult = each_js_1.each.before.cb();
            if (beforeResult instanceof Promise)
                yield beforeResult;
        }
        const cbResult = cb();
        if (cbResult instanceof Promise)
            yield cbResult;
        if (typeof each_js_1.each.after.cb === 'function' && each_js_1.each.after.assert) {
            const afterResult = each_js_1.each.after.cb();
            if (afterResult instanceof Promise)
                yield afterResult;
        }
        if (typeof options.message === 'string') {
            const message = isPoku
                ? `${format_js_1.format.bold(format_js_1.format.success(`✔ ${options.message}`))} ${format_js_1.format.dim(format_js_1.format.success(`› ${FILE}`))}`
                : format_js_1.format.bold(format_js_1.format.success(`✔ ${options.message}`));
            console.log(message);
        }
    }
    catch (error) {
        if (error instanceof node_assert_1.default.AssertionError) {
            const { code, actual, expected, operator } = error;
            const absoultePath = (0, find_file_js_1.findFile)(error).replace(/file:(\/\/)?/, '');
            const file = node_path_1.default.relative(node_path_1.default.resolve(node_process_1.default.cwd()), absoultePath);
            let message = '';
            if (typeof options.message === 'string')
                message = options.message;
            else if (options.message instanceof Error)
                message = options.message.message;
            else if (typeof options.defaultMessage === 'string')
                message = options.defaultMessage;
            /* c8 ignore start */
            const finalMessage = (message === null || message === void 0 ? void 0 : message.trim().length) > 0
                ? format_js_1.format.bold(format_js_1.format.fail(`✘ ${message}`))
                : format_js_1.format.bold(format_js_1.format.fail('✘ No Message'));
            console.log(isPoku
                ? `${finalMessage} ${format_js_1.format.dim(format_js_1.format.fail(`› ${FILE}`))}`
                : finalMessage);
            file && console.log(`${format_js_1.format.dim('      File')} ${file}`);
            console.log(`${format_js_1.format.dim('      Code')} ${code}`);
            console.log(`${format_js_1.format.dim('  Operator')} ${operator}${node_os_1.EOL}`);
            if (!(options === null || options === void 0 ? void 0 : options.hideDiff)) {
                console.log(format_js_1.format.dim(`  ${(options === null || options === void 0 ? void 0 : options.actual) || 'Actual'}:`));
                console.log(format_js_1.format.bold(typeof actual === 'function' || actual instanceof RegExp
                    ? `  ${String(actual)}`
                    : `  ${format_js_1.format.fail(JSON.stringify(actual))}`));
                console.log(`${node_os_1.EOL}  ${format_js_1.format.dim(`${(options === null || options === void 0 ? void 0 : options.expected) || 'Expected'}:`)}`);
                console.log(format_js_1.format.bold(`${typeof expected === 'function' || expected instanceof RegExp
                    ? `  ${String(expected)}`
                    : `  ${format_js_1.format.success(JSON.stringify(expected))}`}`));
            }
            if (options.throw) {
                console.log(error);
                (0, hr_js_1.hr)();
            }
            node_process_1.default.exit(1);
        }
        // Non-assertion errors
        throw error;
    }
    /* c8 ignore stop */
    /* c8 ignore next */
});
exports.parseAssertion = parseAssertion;
