"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.startScript = exports.startService = void 0;
const node_process_1 = __importDefault(require("process"));
const node_child_process_1 = require("child_process");
const runner_js_1 = require("../helpers/runner.js");
const node_path_1 = __importDefault(require("path"));
const list_files_js_1 = require("./list-files.js");
const node_os_1 = require("os");
const format_js_1 = require("../helpers/format.js");
/* c8 ignore start */
const runningProcesses = {};
const secureEnds = () => Object.values(runningProcesses).forEach((end) => end());
const killWindowsProcess = (PID) => (0, node_child_process_1.spawn)('taskkill', ['/F', '/T', '/PID', PID.toString()]);
node_process_1.default.once('SIGINT', () => {
    secureEnds();
});
/* c8 ignore stop */
const backgroundProcess = (runtime, args, file, options) => new Promise((resolve, reject) => {
    let isResolved = false;
    const service = (0, node_child_process_1.spawn)(runtime, args, {
        stdio: ['inherit', 'pipe', 'pipe'],
        /* c8 ignore next */
        shell: runner_js_1.isWindows,
        cwd: (options === null || options === void 0 ? void 0 : options.cwd) ? (0, list_files_js_1.sanitizePath)(node_path_1.default.normalize(options.cwd)) : undefined,
        env: node_process_1.default.env,
        /* c8 ignore next */
        detached: !runner_js_1.isWindows,
        /* c8 ignore next */
        windowsHide: runner_js_1.isWindows,
        timeout: options === null || options === void 0 ? void 0 : options.timeout,
    });
    const PID = service.pid;
    /* c8 ignore start */
    const end = () => {
        delete runningProcesses[PID];
        if (runner_js_1.isWindows) {
            killWindowsProcess(PID);
            return;
        }
        if (['bun', 'deno'].includes(runtime) ||
            ['bun', 'deno'].includes(String(options === null || options === void 0 ? void 0 : options.runner))) {
            node_process_1.default.kill(PID);
            return;
        }
        node_process_1.default.kill(-PID, 'SIGKILL');
        return;
    };
    runningProcesses[PID] = end;
    /* c8 ignore stop */
    service.stdout.on('data', (data) => {
        if (!isResolved && typeof (options === null || options === void 0 ? void 0 : options.startAfter) !== 'number') {
            const stringData = JSON.stringify(String(data));
            if (typeof (options === null || options === void 0 ? void 0 : options.startAfter) === 'undefined' ||
                (typeof (options === null || options === void 0 ? void 0 : options.startAfter) === 'string' &&
                    stringData.includes(options === null || options === void 0 ? void 0 : options.startAfter))) {
                resolve({ end });
                clearTimeout(timeout);
                isResolved = true;
            }
        }
        (options === null || options === void 0 ? void 0 : options.verbose) && console.log(String(data));
    });
    service.stderr.on('data', (data) => {
        if (!isResolved && typeof (options === null || options === void 0 ? void 0 : options.startAfter) !== 'number') {
            const stringData = JSON.stringify(String(data));
            if (typeof (options === null || options === void 0 ? void 0 : options.startAfter) === 'undefined' ||
                (typeof (options === null || options === void 0 ? void 0 : options.startAfter) === 'string' &&
                    stringData.includes(options === null || options === void 0 ? void 0 : options.startAfter))) {
                resolve({ end });
                clearTimeout(timeout);
                isResolved = true;
            }
        }
        (options === null || options === void 0 ? void 0 : options.verbose) && console.log(String(data));
    });
    service.on('error', (err) => {
        secureEnds();
        reject(`Service failed to start: ${err}`);
    });
    service.on('close', (code) => {
        if (code !== 0)
            reject(`Service exited with code ${code}`);
    });
    const timeout = setTimeout(() => {
        if (!isResolved) {
            secureEnds();
            reject(`createService: Timeout\nFile: ${file}`);
        }
    }, (options === null || options === void 0 ? void 0 : options.timeout) || 10000);
    if (typeof (options === null || options === void 0 ? void 0 : options.startAfter) === 'number') {
        setTimeout(() => {
            if (!isResolved) {
                resolve({ end });
                clearTimeout(timeout);
                isResolved = true;
            }
        }, options.startAfter);
    }
});
/**
 *
 * Starts a file in a background process
 *
 * Useful for servers, APIs, etc.
 */
const startService = (file, options) => __awaiter(void 0, void 0, void 0, function* () {
    const runtimeOptions = (0, runner_js_1.runner)(file, { platform: options === null || options === void 0 ? void 0 : options.platform });
    const runtime = runtimeOptions.shift();
    const runtimeArgs = [...runtimeOptions, file];
    return yield backgroundProcess(runtime, runtimeArgs, node_path_1.default.normalize((0, list_files_js_1.sanitizePath)(file)), options);
});
exports.startService = startService;
/**
 *
 * Starts a script (package.json) or task (deno.json) in a background process
 *
 * By default, it uses **npm**, but you can costumize it using the `runner` option.
 *
 * Useful for servers, APIs, etc.
 *
 * ---
 *
 * `startScript` currently doesn't works for **Bun** and **Deno**.
 *
 * - See: https://github.com/wellwelwel/poku/issues/143
 */
const startScript = (script, options) => __awaiter(void 0, void 0, void 0, function* () {
    const runner = (options === null || options === void 0 ? void 0 : options.runner) || 'npm';
    const runtimeOptions = (0, runner_js_1.scriptRunner)(runner);
    const runtime = runtimeOptions.shift();
    const runtimeArgs = [...runtimeOptions, script];
    /* c8 ignore start */
    if (['bun', 'deno'].includes(runner))
        throw new Error(`${format_js_1.format.bold('startScript')} currently doesn't works for Bun and Deno.${node_os_1.EOL}See: https://github.com/wellwelwel/poku/issues/143`);
    /* c8 ignore stop */
    return yield backgroundProcess(runtime, runtimeArgs, script, Object.assign(Object.assign({}, options), { isScript: true, runner }));
});
exports.startScript = startScript;
